import os
import pwd
import subprocess
import yaml
import re
from jinja2 import Template
import shutil
import fnmatch
import time
from urllib.parse import urlparse

from charmhelpers import fetch
from charmhelpers.core import host, hookenv, unitdata
from charmhelpers.core.templating import render
from charmhelpers.contrib.charmsupport import nrpe
from charms.reactive import (
    when, when_not, set_state, remove_state, is_state, hook
)
from charms.reactive.helpers import any_file_changed, data_changed
from charms.layer import snap


PROMETHEUS_YML_TMPL = 'prometheus.yml.j2'
PROMETHEUS_DEF_TMPL = 'etc_default_prometheus.j2'
PROMETHEUS_RULES_CRON = '/etc/cron.d/juju-prometheus-rules-refresh'
PROMETHEUS_SSH_KEYFILE = '/root/.ssh/prometheus_git_key'
REFRESH_RULES_SCRIPT = '/usr/local/bin/refresh_rules.sh'
REFRESH_RULES_SCRIPT_TMPL = 'refresh_rules.sh.j2'


def templates_changed(tmpl_list):
    return any_file_changed(['templates/{}'.format(x) for x in tmpl_list])


def filter_conflicting_daemon_arg(arg):
    return arg  # Just a stub function for now.


def parse_daemon_args_string(daemon_args):
    return list(filter(filter_conflicting_daemon_arg, daemon_args.split(";")))


def get_daemon_args_list_from_config():
    config = hookenv.config()
    daemon_args = config.get('daemon-args')
    assert isinstance(daemon_args, str)

    if daemon_args:
        hookenv.log("Retrieving {} from config".format(daemon_args),
                    level="DEBUG")
        return parse_daemon_args_string(daemon_args)
    else:
        hookenv.log("No daemon args in config", level="DEBUG")
        return []


# This could probably be a bit more intelligent.
# Curently we just return the processed list of
# string of args.
def daemon_args():
    args_list = get_daemon_args_list_from_config()
    return sorted(args_list)


def configure_rules():
    config = hookenv.config()
    # Static rules provided as config options
    if config.get('custom-rules', False):
        with open(get_paths()['custom_rules_path'], 'w') as fh:
            fh.write(config['custom-rules'])

    kv = unitdata.kv()
    if config.get('rules-repo', False) or kv.get('prometheus.rules-repo') != config.get('rules-repo'):
        kv.set('prometheus.rules-repo', config.get('rules-repo', False))
        # Remove files on disk if rules-repo is empty or has changed
        remove_external_rules()
        set_state('prometheus.do-reconfig-yml')

    if config.get('rules-repo', False):
        ssh_command = ''
        if config.get('rules-update-ssh-key', False):
            host.mkdir(os.path.dirname(PROMETHEUS_SSH_KEYFILE), perms=0o700)
            with open(PROMETHEUS_SSH_KEYFILE, 'w') as f:
                f.write(config['rules-update-ssh-key'])
            os.chmod(PROMETHEUS_SSH_KEYFILE, 0o600)
            ssh_command = 'ssh -i {}'.format(PROMETHEUS_SSH_KEYFILE)

        rules_path = get_paths()['external_rules_path']
        if os.path.exists(rules_path):
            subprocess.check_call(['git', '-C', rules_path, 'pull'],
                                  env={'GIT_SSH_COMMAND': ssh_command})
        else:
            # New repo, store key during initial clone
            subprocess.check_call(['git', 'clone', config['rules-repo'], rules_path],
                                  env={'GIT_SSH_COMMAND': ssh_command + ' -o StrictHostKeyChecking=no'})

        options = {
            'prometheus_yml': get_paths()['prometheus_yml'],
            'external_rules_path': get_paths()['external_rules_path'],
            'promtool': get_paths()['promtool'],
            'ssh_command': ssh_command,
            'rules_update_notify_email': config.get('rules-update-notify-email', False),
        }
        render(source=REFRESH_RULES_SCRIPT_TMPL,
               target=REFRESH_RULES_SCRIPT,
               context=options,
               perms=0o755,
               )

        schedule = config.get('rules-update-schedule', '*/10 * * * *')
        with open(PROMETHEUS_RULES_CRON, 'w') as f:
            f.write('{} root {} &>/dev/null\n'.format(schedule, REFRESH_RULES_SCRIPT))
        set_state('prometheus.do-reconfig-yml')


def remove_external_rules():
    if os.path.exists(PROMETHEUS_RULES_CRON):
        os.unlink(PROMETHEUS_RULES_CRON)
    if os.path.exists(PROMETHEUS_SSH_KEYFILE):
        os.unlink(PROMETHEUS_SSH_KEYFILE)
    if os.path.exists(get_paths()['external_rules_path']):
        shutil.rmtree(get_paths()['external_rules_path'])


def get_rule_files():
    config = hookenv.config()
    rule_files = []
    if config.get('custom-rules', False):
        rule_files.append(get_paths()['custom_rules_path'])

    for root, dirs, filenames in os.walk(get_paths()['external_rules_path']):
        for filename in fnmatch.filter(filenames, '*.rules'):
            rule_files.append(os.path.join(root, filename))
    rule_files.sort()
    return rule_files


# TODO: once there's reactive support for storage hooks, convert off @hook()
@hook('metrics-{filesystem,block}-storage-attached')
def configure_storage():
    storage_path = subprocess.check_output(
        ['storage-get', 'location']).decode().strip()
    kv = unitdata.kv()
    kv.set('storage-path', storage_path)
    runtime_args('-storage.local.path', storage_path)
    set_state('storage.configured')
    set_state('prometheus.do-check-reconfig')


@hook('install')
def install():
    set_state('prometheus.do-install')


def set_datadir_perms():
    datadir = unitdata.kv().get('storage-path', False)
    if not datadir:
        # No juju storage attached, use defaults from package
        return
    users = [i for i in pwd.getpwall() if i.pw_name == 'nobody']
    if len(users) == 1:
        os.lchown(datadir, users[0].pw_uid, users[0].pw_gid)


@when('prometheus.do-install')
def install_packages():
    hookenv.status_set('maintenance', 'Installing software')
    pkgname = 'prometheus'
    promreg_pkgname = 'promreg'

    config = hookenv.config()
    source = config.get('install_method', 'snap')
    channel = config.get('snap_channel', 'stable')

    kv = unitdata.kv()
    if not kv.get('install_method', False):
        kv.set('install_method', source)
    elif kv.get('install_method') != source:
        hookenv.status_set('blocked', 'Install method changes are not supported, '
                                      'revert install_method option to previous value')
        return

    if not os.path.exists('/usr/sbin/sendmail'):
        fetch.apt_install('sendmail-bin')  # Needed for cronjob email notifications
    if source == 'snap':
        snap.install(pkgname, channel=channel, force_dangerous=False)
        if config.get('prometheus_registration_listen', False):
            snap.install(promreg_pkgname, channel=channel, force_dangerous=False)
            subprocess.check_call(['snap', 'connect', 'prometheus:content',
                                   'promreg:content'])
        remove_state('prometheus.do-install')
        set_state('prometheus.do-restart')
        hookenv.status_set('maintenance', 'Software installed')
    elif source == 'apt':
        fetch.configure_sources()
        fetch.apt_update()
        fetch.apt_install(pkgname)
        if config.get('prometheus_registration_listen', False):
            fetch.apt_install(promreg_pkgname)
        remove_state('prometheus.do-install')
        set_state('prometheus.do-restart')
        hookenv.status_set('maintenance', 'Software installed')
    else:
        hookenv.status_set('blocked',
                           'Unsupported install_method: {}'.format(source))


def runtime_args(key=None, value=None):
    kv = unitdata.kv()
    args = kv.get('runtime_args', {})
    if key:
        args.update({key: value})
        kv.set('runtime_args', args)
    args_list = ['{} {}'.format(k, v) for k, v in args.items() if v] + \
        daemon_args()
    # sorted list is needed to avoid data_changed() false-positives
    return sorted(args_list)


def get_paths():
    config = hookenv.config()
    source = config.get('install_method', 'snap')
    if source == 'snap':
        return {
                'prometheus_yml': '/var/snap/prometheus/current/prometheus.yml',
                'prometheus_def': '/var/snap/prometheus/current/daemon_arguments',
                'prometheus_promreg_tgts_yml': '/var/snap/prometheus/current/promreg/targets.yaml',
                'custom_rules_path': '/var/snap/prometheus/current/custom.rules',
                'external_rules_path': '/var/snap/prometheus/current/custom.rules.d',
                'promtool': '/snap/bin/prometheus.promtool',
                'service_name': 'snap.prometheus.prometheus',
                'promreg_yml': '/var/snap/promreg/common/promreg.yaml',
                'promreg_tgts_yml': '/var/snap/promreg/common/promreg/targets.yaml',
                'promreg_service_name': 'snap.promreg.promreg'
                }
    else:
        return {
                'prometheus_yml': '/etc/prometheus/prometheus.yml',
                'prometheus_def': '/etc/default/prometheus',
                'prometheus_promreg_tgts_yml': '/etc/prometheus/promreg/targets.yaml',
                'custom_rules_path': '/etc/prometheus/custom.rules',
                'external_rules_path': '/etc/prometheus/custom.rules.d',
                'promtool': 'promtool',
                'service_name': 'prometheus',
                'promreg_yml': '/etc/prometheus/promreg.yaml',
                'promreg_tgts_yml': '/etc/prometheus/promreg/targets.yaml',
                'promreg_service_name': 'promreg'
                }


def validate_config():
    subprocess.check_call([get_paths()['promtool'], 'check-config', get_paths()['prometheus_yml']])


def get_version():
    """Extract version info
    Defaults to 1.0.0 which guarantees stable API
    """
    pattern = b'.*version (\d+)\.(\d+)\.(\d+) .*$'
    output = subprocess.check_output([get_paths()['promtool'], 'version']).split(b'\n')[0]
    if re.match(pattern, output):
        major, minor, patch = re.match(pattern, output).groups()
        return (int(major), int(minor), int(patch))
    else:
        return (1, 0, 0)


def get_manual_jobs():
    config = hookenv.config()
    if not config.get('scrape-jobs', False):
        return None
    try:
        yaml.safe_load(config.get('scrape-jobs'))
    except Exception:
        return None
    else:
        snmp_exporter = unitdata.kv().get('snmp_exporter_address', '127.0.0.1:9116')
        blackbox_exporter = unitdata.kv().get('blackbox_exporter_address', '127.0.0.1:9115')
        t = Template(config.get('scrape-jobs'))
        return t.render(snmp_exporter=snmp_exporter, blackbox_exporter=blackbox_exporter)


@when('prometheus.do-reconfig-yml')
def write_prometheus_config_yml():
    hookenv.status_set('maintenance', 'Updating configuration')
    config = hookenv.config()

    ver_major, ver_minor, ver_patch = get_version()
    target_jobs = unitdata.kv().get('target_jobs', [])
    scrape_jobs = unitdata.kv().get('scrape_jobs', [])
    manual_jobs = get_manual_jobs()

    # transform eg. 'h1:p1 ,  h2:p2' (string), to ['h1:p1', 'h2:p2'] (list)
    static_targets = None
    if config.get('static-targets'):
        static_targets = [x.strip()
                          for x in config.get('static-targets', '').split(',')]
    federate_config = None
    if config.get('federate-config'):
        federate_config = yaml.safe_load(config.get('federate-config')) or None

    monitor_name = '{}-monitor'.format(hookenv.service_name())
    if config.get('monitor-name', False):
        monitor_name = config.get('monitor-name')

    # convert e.g. 'l1=v1,l2=v2' in {'l1': 'v1, 'l2'; 'v2}
    external_labels_config = config.get('external-labels', '').replace(' ', '')
    if external_labels_config:
        external_labels = dict(
            entry.split('=') for entry in external_labels_config.split(','))
    else:
        external_labels = {}

    prom_metrics_path = "/metrics"
    external_url = config.get("external_url", "")
    if external_url:
        ext_url = urlparse(external_url)
        prom_metrics_path = '{}/metrics'.format(ext_url.path.strip('/'))

    options = {
        'scrape_interval': config['scrape-interval'],
        'evaluation_interval': config['evaluation-interval'],
        'static_targets': static_targets,
        'private_address': hookenv.unit_get('private-address'),
        'listen_port': config['web-listen-port'],
        'promreg': config.get('prometheus_registration_listen', False),
        'promreg_tgts_yml': get_paths()['promreg_tgts_yml'],
        'monitor_name': monitor_name,
        'prom_metrics_path': prom_metrics_path,
        'external_labels': external_labels,
        'rule_files': get_rule_files(),
        'prometheus_promreg_tgts_yml': get_paths()['prometheus_promreg_tgts_yml'],
        'prometheus_registration_listen': config.get('prometheus_registration_listen', ''),
        'prometheus_registration_port': config.get('prometheus_registration_port', ''),
        'prometheus_registration_authtoken': config.get('prometheus_registration_authtoken', ''),
        'jobs': target_jobs,
        'federate': federate_config,
        'scrape_jobs': scrape_jobs,
        'manual_jobs': manual_jobs,
        'ver_major': ver_major,
        'ver_minor': ver_minor,
        'ver_patch': ver_patch,
        'label_juju_units': config.get('label-juju-units', False),
    }

    render(source=PROMETHEUS_YML_TMPL,
           target=get_paths()['prometheus_yml'],
           context=options
           )
    validate_config()

    if config.changed('prometheus_registration_listen'):
        render(source='promreg.yaml.j2',
               target=get_paths()['promreg_yml'],
               context=options,
               perms=0o440
               )
        hookenv.open_port(config['prometheus_registration_port'])
        set_state('promreg.do-restart')

    set_state('prometheus.do-reload')
    remove_state('prometheus.do-reconfig-yml')


def check_ports(new_port):
    kv = unitdata.kv()
    if kv.get('prometheus.port') != new_port:
        hookenv.open_port(new_port)
        if kv.get('prometheus.port'):  # Dont try to close non existing ports
            hookenv.close_port(kv.get('prometheus.port'))
        kv.set('prometheus.port', new_port)


@when('prometheus.do-reconfig-def')
def write_prometheus_config_def():
    hookenv.status_set('maintenance', 'Updating configuration')
    config = hookenv.config()
    port = config.get('web-listen-port', 9090)
    check_ports(port)
    kv = unitdata.kv()
    if config.get('external_url', False):
        vars = {
            'private_address': hookenv.unit_get('private-address'),
            'public_address': hookenv.unit_get('public-address'),
            # prometheus default:
            'port': port,
        }
        runtime_args('-web.external-url',
                     config['external_url'].format(**vars))
    runtime_args('-web.listen-address',
                 ":{}".format(kv.get('prometheus.port')))
    args = runtime_args()
    hookenv.log('runtime_args: {}'.format(args))
    if args:
        render(source=PROMETHEUS_DEF_TMPL,
               target=get_paths()['prometheus_def'],
               context={'args': args},
               )
    set_state('prometheus.do-restart')
    remove_state('prometheus.do-reconfig-def')


@when_not('prometheus.started')
def setup_prometheus():
    if is_state('prometheus.do-install'):
        hookenv.status_set('waiting',
                           'Waiting for installation to complete')
        return
    hookenv.status_set('maintenance', 'Configuring software')
    set_datadir_perms()
    install_packages()
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
def check_config():
    set_state('prometheus.do-check-reconfig')


@when('prometheus.do-check-reconfig')
@when_not('prometheus.do-install')
def check_reconfig_prometheus():
    config = hookenv.config()
    kv = unitdata.kv()
    target_jobs = kv.get('target_jobs', [])
    scrape_jobs = kv.get('scrape_jobs', [])
    args = runtime_args()
    install_opts = ('install_sources', 'install_keys', 'snap_channel',
                    'prometheus_registration_listen')
    def_change_opts = ('daemon-args', 'external_url', 'web-listen-port')
    rules_change_opts = ('custom-rules', 'rules-repo', 'rules-update-schedule', 'rules-update-ssh-key',
                         'rules-update-notify-email')
    yml_change_opts = set(config.keys()) - set(install_opts + def_change_opts + rules_change_opts)
    yml_change_opts = yml_change_opts.union(['external_url', 'web-listen-port'])
    if any(config.changed(opt) for opt in install_opts):
        set_state('prometheus.do-install')
    if any(config.changed(opt) for opt in def_change_opts):
        hookenv.log('One of options affecting daemon args has changed, scheduling reconfiguration')
        set_state('prometheus.do-reconfig-def')
    if any(config.changed(opt) for opt in rules_change_opts):
        configure_rules()
    if any(config.changed(opt) for opt in yml_change_opts):
        set_state('prometheus.do-reconfig-yml')
    if any((
        data_changed('prometheus.target_jobs', target_jobs),
        data_changed('prometheus.scrape_jobs', scrape_jobs),
        templates_changed([PROMETHEUS_YML_TMPL]),
    )):
        set_state('prometheus.do-reconfig-yml')
    if data_changed('prometheus.args', args):
        hookenv.log('CLI flags have change, scheduling reconfiguration')
        set_state('prometheus.do-reconfig-def')
    if templates_changed([PROMETHEUS_DEF_TMPL]):
        hookenv.log('Daemon flags template changed, scheduling reconfiguration')
        set_state('prometheus.do-reconfig-def')
    if templates_changed([REFRESH_RULES_SCRIPT_TMPL]):
        configure_rules()
    # Cron can update rules on disk, update-status hook can trigger config update
    if kv.get('prometheus.rule_files') != get_rule_files():
        kv.set('prometheus.rule_files', get_rule_files())
        set_state('prometheus.do-reconfig-yml')

    remove_state('prometheus.do-check-reconfig')


@when('prometheus.do-restart')
@when_not('prometheus.do-install')
def restart_prometheus():
    if not host.service_running(get_paths()['service_name']):
        hookenv.log('Starting {}...'.format(get_paths()['service_name']))
        host.service_start(get_paths()['service_name'])
    else:
        hookenv.log('Restarting {}, config file changed...'.format(get_paths()['service_name']))
        host.service_restart(get_paths()['service_name'])
        hookenv.log('Sleeping for 30s to make sure prometheus has time to start. '
                    'Needed in case of multiple configuration changes made at short intervals')
        time.sleep(30)

    hookenv.status_set('active', 'Ready')
    set_state('prometheus.started')
    remove_state('prometheus.do-restart')
    remove_state('prometheus.do-reload')  # We've just restarted, no need to reload


@when('prometheus.started')
@when('prometheus.do-reload')
@when_not('prometheus.do-install')
@when_not('prometheus.do-restart')
def reload_prometheus():
    if host.service_running(get_paths()['service_name']):
        # Use SIGHUP to reload config gracefully
        subprocess.check_call(['killall', '-HUP', 'prometheus'])
        remove_state('prometheus.do-reload')
        hookenv.status_set('active', 'Ready')
    else:
        # Let's make sure prometheus is up
        set_state('prometheus.do-restart')


@when('promreg.do-restart')
def restart_promreg():
    hookenv.log('Restarting promreg')
    if not host.service_running(get_paths()['promreg_service_name']):
        host.service_start(get_paths()['promreg_service_name'])
    else:
        hookenv.log('Restarting {}, config file changed...'.format(get_paths()['promreg_service_name']))
        host.service_restart(get_paths()['promreg_service_name'])
    remove_state('promreg.do-restart')


# Relations
@when('prometheus.started')
@when_not('target.available')
def update_prometheus_no_targets():
    unitdata.kv().set('target_jobs', [])
    data_changed('target.related_services', [])
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when_not('scrape.available')
def update_prometheus_no_scrapes():
    unitdata.kv().set('scrape_jobs', [])
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when('target.available')
def update_prometheus_targets(target):
    unit_names = get_related_unit_names()
    services = target.services()
    related_targets = []
    for service in sorted(services, key=lambda k: k['service_name']):
        targets = []
        for unit in service['hosts']:
            hookenv.log('{} has a unit {}:{}'.format(
                service['service_name'],
                unit['hostname'],
                unit['port']), level='debug')
            hostname = '{hostname}:{port}'.format(**unit)
            targets.append((hostname, unit_names.get(hostname)))
        related_targets.append({'job_name': service['service_name'],
                                'targets': sorted(targets)})

    unitdata.kv().set('target_jobs', related_targets)
    set_state('prometheus.do-check-reconfig')


def get_related_unit_names():
    '''Return a dict mapping unit hostnames to names.'''
    app_units = {}
    for relation_id in hookenv.relation_ids('target'):
        for relation_data in hookenv.relations_for_id(relation_id):
            hostname = relation_data.get('hostname')
            port = relation_data.get('port')
            if not (hostname and port):
                continue
            hostname = '{}:{}'.format(hostname, port)
            unit = relation_data['__unit__']
            # The name of the principal unit in case the remote end is a
            # subordinate charm.
            principal_unit = hookenv.relation_get(
                'principal-unit', unit, relation_id)
            app_units[hostname] = principal_unit or unit
    return app_units


@when('prometheus.started')
@when('scrape.available')
def update_prometheus_scrape_targets(target):
    targets = target.targets()
    unitdata.kv().set('scrape_jobs', targets)
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when_not('alertmanager-service.available')
def update_prometheus_no_alertmanager():
    runtime_args('-alertmanager.url', None)
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when('alertmanager-service.available')
def update_prometheus_alertmanager(alertmanager):
    services = alertmanager.services()
    if not (data_changed('alertmanager-service.related_services', services)):
        return
    # XXX: as of prometheus 0.17, it can only point to a single alertmanager,
    #      last one from below loop will be used.
    for service in services:
        for unit in service['hosts']:
            hookenv.log('{} has a unit {}:{}'.format(
                service['service_name'],
                unit['hostname'],
                unit['port']))
            runtime_args('-alertmanager.url',
                         'http://{hostname}:{port}'.format(**unit))
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when_not('snmp-exporter.available')
def update_prometheus_no_snmp_exporter():
    unitdata.kv().unset('snmp_exporter_address')
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when_not('blackbox-exporter.available')
def update_prometheus_no_blackbox_exporter():
    unitdata.kv().unset('blackbox_exporter_address')
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when('snmp-exporter.available')
def update_prometheus_snmp_exporter(snmp_exporter):
    services = snmp_exporter.services()
    if not (data_changed('snmp-exporter.related_services', services)):
        return
    # At this point we only support single snmp exporter target
    for service in services:
        for unit in service['hosts']:
            hookenv.log('{} has a unit {}:{}'.format(
                        service['service_name'], unit['hostname'], unit['port']))
            unitdata.kv().set('snmp_exporter_address', '{hostname}:{port}'.format(**unit))
    set_state('prometheus.do-check-reconfig')


@when('prometheus.started')
@when('blackbox-exporter.available')
def update_prometheus_blackbox_exporter(blackbox_exporter):
    services = blackbox_exporter.services()
    if not (data_changed('blackbox-exporter.related_services', services)):
        return
    # At this point we only support single blackbox exporter target
    for service in services:
        for unit in service['hosts']:
            hookenv.log('{} has a unit {}:{}'.format(
                        service['service_name'], unit['hostname'], unit['port']))
            unitdata.kv().set('blackbox_exporter_address', '{hostname}:{port}'.format(**unit))
    set_state('prometheus.do-check-reconfig')


@hook('upgrade-charm')
def upgrade_charm():
    remove_state('nagios-nrpe.configured')


@when('config.changed')
def config_changed():
    remove_state('nagios-nrpe.configured')


@when('prometheus.started')
@when('nrpe-external-master.available')
@when_not('nagios-nrpe.configured')
def update_nrpe_config(svc):
    hostname = nrpe.get_nagios_hostname()
    nrpe_setup = nrpe.NRPE(hostname=hostname)
    port = unitdata.kv().get('prometheus.port', 9090)

    config = hookenv.config()
    graph_path = "/graph"
    external_url = config.get("external_url", "")
    if external_url:
        ext_url = urlparse(external_url)
        graph_path = '/{}/graph'.format(ext_url.path.strip('/'))

    nrpe_setup.add_check('prometheus_http',
                         'Prometheus HTTP check',
                         'check_http -I 127.0.0.1 -p {} -u {}'.format(port, graph_path)
                         )
    nrpe_setup.write()
    set_state('nagios-nrpe.configured')


@when('prometheus.started')
@when('grafana-source.available')
def provide_grafana_source(grafana):
    kv = unitdata.kv()
    port = kv.get('prometheus.port')
    grafana.provide('prometheus', port, 'Juju generated source')


@when('prometheus.started')
@when('website.available')
def configure_website(website):
    port = unitdata.kv().get('prometheus.port', 9090)
    website.configure(port=port)
